/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.corba.trader;

// Package dependencies
import org.objectweb.openccm.command.lib.ApplicationBase;
import org.objectweb.openccm.corba.trader.parser.Services;
import org.objectweb.openccm.corba.trader.parser.Service;
import org.objectweb.openccm.corba.trader.parser.ServicesUnmarshaller;
import org.objectweb.openccm.corba.TheTradingService;

/**
 * Implementation of the str_feed command.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 *
 * @version 1.0
 */

public class TypeRepositoryFeed 
     extends ApplicationBase
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected ServiceTypeActions _actions;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    TypeRepositoryFeed()
    {
        // Calls the CompilerBase constructor.
        super(new org.objectweb.util.cmdline.lib.DefaultCommandLine("trader_feed",
                                     "xml_file",
                                     "Feed the CORBA Trader Service Type Repository with an XML service types file.",
                                     true));

    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     * @param repos The service type repository in which the new
     * service types will be registered.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        return run(args, TheTradingService.getRepository());
    }

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args, org.omg.CosTradingRepos.ServiceTypeRepository repos)
    {
        String filename = args[0];
        boolean failed = false;

        if (!new java.io.File(filename).exists())
            getConsole().error("File " + filename + " not found!");

        java.io.File file = new java.io.File(filename);
        getConsole().message("Reading from file " + filename + "...");

        // Compiles the file.
        Services services;
        try {
            services = ServicesUnmarshaller.unmarshal(file);
        } catch (java.io.IOException ex) {
            getConsole().error("Problem when unmarshalling " + filename);
            return -1;
        }
        if(services == null) return -1;
        _actions = new ServiceTypeActions(repos);

        getConsole().message("Compiling " + filename + " file...");
        java.util.List list = services.getServiceList();
        for (java.util.Iterator iter = list.iterator() ; iter.hasNext() ; )
        {
            try {
                _actions.create_service((Service)iter.next());
            } catch (org.omg.CosTrading.IllegalServiceType ex) {
                getConsole().error("Bad service type name '" + 
                                   ex.type + "'.");
                failed=true;
            } catch (org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.ServiceTypeExists ex) {
                getConsole().error("Service definition '" + 
                                   ex.name + 
                                   "' already exist.");
                failed=true;
            } catch (org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.InterfaceTypeMismatch ex) {
                getConsole().error("Invalid service OMG IDL interface: '" + 
                                   ex.derived_if + 
                                   "' in '" +
                                   ex.derived_service + "'.");
                failed=true;
            } catch (org.omg.CosTrading.IllegalPropertyName ex) {
                getConsole().error("Bad service property name: '" +
                                   ex.name + "'.");
                failed=true;
            } catch (org.omg.CosTrading.DuplicatePropertyName ex) {
                getConsole().error("Already defined property name: '" +
                                   ex.name + "'.");
                failed=true;
            } catch (org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.ValueTypeRedefinition ex) {
                getConsole().error("Property value type redefinition between '" +
                                   ex.type_1 + "' and '" + 
                                   ex.type_2 + "' in property '" +
                                   ex.definition_1.name + "'.");
                failed=true;
            } catch (org.omg.CosTrading.UnknownServiceType ex) {
                getConsole().error("Unknown service definition: '" + 
                                   ex.type + "'.");
                failed=true;
            } catch (org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.DuplicateServiceTypeName ex) {
                getConsole().error("Duplicate super type: '" + ex.name + "'.");
                failed=true;
            }
        }

        // At least one the services cannot compile
        if (failed) {
            getConsole().message("Compilation failed.");
            return -1;
        }

        // All is OK.
        getConsole().message("Compilation completed.");
        return 0;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        TypeRepositoryFeed trfeed = new TypeRepositoryFeed();
        trfeed.runMain(args);
    }
}
