/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 mai 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.OpenCCM;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.zip.ZipFile;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.FilterSet;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.ui.dialogs.IOverwriteQuery;
import org.eclipse.ui.wizards.datatransfer.FileSystemStructureProvider;
import org.eclipse.ui.wizards.datatransfer.ImportOperation;
import org.eclipse.ui.wizards.datatransfer.ZipFileStructureProvider;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.BuildPropertySet;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.CPManager;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.ProjectMessages;

/**
 * This class implements the operations to execute 
 * when project configuration is finished
 *
 * @author offroy@lifl.fr
 *
 * @version 0.1
 */
public class ProjectCreationOperation implements IRunnableWithProgress {

    private IOverwriteQuery fOverwriteQuery;
    private IResource fElementToOpen;
    private ProjectCreationWizardPage[] fPages;

    /**
     * @param pages
     * @param overwriteQuery
     */
    public ProjectCreationOperation(
        ProjectCreationWizardPage[] pages,
        IOverwriteQuery overwriteQuery) {
        fElementToOpen = null;
        fPages = pages;
        fOverwriteQuery = overwriteQuery;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void run(IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        if (monitor == null) {
            monitor = new NullProgressMonitor();
        }
        try {
            monitor.beginTask(
                ProjectMessages.getString(
                    "ProjectCreationOperation.op_desc.ORBacus"),
                fPages.length);
            IWorkspaceRoot root = MainPlugin.getWorkspace().getRoot();

            for (int i = 0; i < fPages.length; i++) {
                createProject(
                    root,
                    fPages[i],
                    new SubProgressMonitor(monitor, 1));
            }
        } finally {
            monitor.done();
        }
    }

    /**
     * @param root
     * @param page
     * @param monitor
     * @throws InvocationTargetException
     * @throws InterruptedException
     */
    private void createProject(
        IWorkspaceRoot root,
        ProjectCreationWizardPage page,
        IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        IConfigurationElement desc =
            page.getAttributeManager().getConfigurationElement();

        IConfigurationElement[] imports = desc.getChildren("import");
        IConfigurationElement[] natures = desc.getChildren("nature");
        IConfigurationElement[] references = desc.getChildren("references");
        int nImports = (imports == null) ? 0 : imports.length;
        int nNatures = (natures == null) ? 0 : natures.length;
        int nReferences = (references == null) ? 0 : references.length;

        monitor.beginTask(
            ProjectMessages.getString("ProjectCreationOperation.op_desc_proj"),
            nImports + 1);

        //String name = page.getName();
        String name = page.getProjectChooser().get_ProjectName();
        String[] natureIds = new String[nNatures];
        for (int i = 0; i < nNatures; i++) {
            natureIds[i] = natures[i].getAttribute("id");
        }
        IProject[] referencedProjects = new IProject[nReferences];
        for (int i = 0; i < nReferences; i++) {
            referencedProjects[i] =
                root.getProject(references[i].getAttribute("id"));
        }

        IProject proj =
            configNewProject(
                root,
                name,
                natureIds,
                referencedProjects,
                monitor);

        for (int i = 0; i < nImports; i++) {
            doImports(proj, imports[i], new SubProgressMonitor(monitor, 1));
        }

        String open = desc.getAttribute("open");
        if (open != null && open.length() > 0) {
            IResource fileToOpen = proj.findMember(new Path(open));
            if (fileToOpen != null) {
                fElementToOpen = fileToOpen;
            }
        }

        // generate the build.properties file for OpenCCM
        generateFile(
            proj,
            page,
            monitor,
            desc.getAttribute("BuildPropertiesFileLocation"),
            desc.getAttribute("BuildPropertiesFileName"),
            page.getOpenccmConfigChooser().getCheckGroup().isSubGroupVisible());

        // import the OpenCCM src into the project
        importOpenCCMsrc(page, proj, monitor);

        configureEclipse(page, proj);

        // create Classpath for Eclipse
        CPManager.setEclipseCP(page, monitor);
    }

    /**
     * @param page
     * @param proj
     * @param monitor
     */
    private void importOpenCCMsrc(
        ProjectCreationWizardPage page,
        IProject proj,
        IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        // TODO make a copy not an import (because of date file changes for CVS use) 
        if (page.getCvsFSchooser().getCheckGroup().isSubGroupVisible()) {
			Project antProject = new Project();
			antProject.init();
			Copy copy = new Copy();
			copy.setProject(antProject);
			FileSet tmpSRC = new FileSet();
			tmpSRC.setDir(new File(page.getCvsFSchooser().get_CVSsrcFSPathValue()));
			copy.addFileset(tmpSRC);
			copy.setTodir(proj.getLocation().toFile());
			copy.setIncludeEmptyDirs(true);
			copy.setOverwrite(true);
			copy.setVerbose(true);
			copy.setPreserveLastModified(true);
			copy.perform();			

        	//copyFilesFromFS();
            /*importFilesFromFS(
                new File(page.getCvsFSchooser().get_CVSsrcFSPathValue()),
                proj.getFullPath(),
                new SubProgressMonitor(monitor, 1));*/
                try {
					proj.refreshLocal(IResource.DEPTH_INFINITE,new SubProgressMonitor(monitor, 1));
				} catch (CoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
        }

    }

    /**
     * @param page
     */
    private void configureEclipse(
        ProjectCreationWizardPage page,
        IProject proj) {
        if (page
            .getEclipseConfigChooser()
            .getCheckGroup()
            .isSubGroupVisible()) {
            if (page
                .getEclipseConfigChooser()
                .get_ImportWorkbenchPreference()
                .isEnable()) {
                String outDirectory =
                    page
                        .getEclipseConfigChooser()
                        .get_ImportWorkbenchPreference()
                        .get_Name();

                if (new File(outDirectory).exists())
                    backupWorkbenchPreference(
                        new Path(
                            new Path(outDirectory)
                                .addTrailingSeparator()
                                .toOSString()
                                + "eclipseBackup.epf"));

                importWorkbenchPreference(
                    new Path(
                        proj.getLocation().addTrailingSeparator().toOSString()
                            + "eclipse_Config.epf"));
            }
        }
    }

    /**
     * @param proj
     * @param page
     * @param monitor
     * @param fileLocation
     * @param fileName
     * @param generate
     */
    private void generateFile(
        IProject proj,
        ProjectCreationWizardPage page,
        IProgressMonitor monitor,
        String fileLocation,
        String fileName,
        boolean generate) {
        if (!generate)
            return;

        BuildPropertySet buildConfig = new BuildPropertySet(page);

        //TODO remove debug
        //System.out.println(buildConfig.showDebug(generate));

        try {
            Project antProject = new Project();
            antProject.init();
            Copy buildCopy = new Copy();
            buildCopy.setProject(antProject);
            File in = null;
            String inBuildPropertiesFile = fileLocation;
            String buildPropertiesFileName = fileName;
            if (inBuildPropertiesFile != null
                && buildPropertiesFileName != null) {
                in = getFSFromPluginDir(inBuildPropertiesFile);
                buildCopy.setFile(in.getAbsoluteFile());
                File out =
                    new File(
                        proj.getLocation().toFile(),
                        buildPropertiesFileName);
                buildCopy.setTofile(out.getAbsoluteFile());
                FilterSet fs = buildCopy.createFilterSet();
                buildConfig.createOpenCCMFilter(fs);
                buildCopy.perform();
                proj.refreshLocal(
                    IResource.DEPTH_INFINITE,
                    new SubProgressMonitor(monitor, 1));
            }
        } catch (CoreException e) {
            // TODO add somethig to this catch block
            e.printStackTrace();
        }

    }

    /**
     * @param string
     */
    private void backupWorkbenchPreference(IPath exportFile) {
        // TODO Auto-generated method stub
        try {
            Preferences.exportPreferences(exportFile);
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * import Eclipse Workbench preference from importFile
     * @param importFile
     */
    private void importWorkbenchPreference(IPath importFile) {
        // import Eclipse Workbench preference from file represented by importFile
        try {
            Preferences.importPreferences(importFile);
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * @param project
     * @param curr
     * @param monitor
     * @throws InvocationTargetException
     * @throws InterruptedException
     */
    private void doImports(
        IProject project,
        IConfigurationElement curr,
        SubProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        try {
            IPath destPath;
            String name = curr.getAttribute("dest");
            if (name == null || name.length() == 0) {
                destPath = project.getFullPath();
            } else {
                IFolder folder = project.getFolder(name);
                if (!folder.exists()) {
                    folder.create(true, true, null);
                }
                destPath = folder.getFullPath();
            }

            String importPath = curr.getAttribute("src");
            if (importPath == null) {
                importPath = "";
                MainPlugin.log("projectsetup descriptor: import missing");
                return;
            }
            // Zip or FileSystem Import
            String typeImport = curr.getAttribute("type");
            if (typeImport == null) {
                typeImport = "zip";
            }

            if (typeImport.compareTo("zip") == 0) {
                ZipFile zipFile = getZipFileFromPluginDir(importPath);
                importFilesFromZip(
                    zipFile,
                    destPath,
                    new SubProgressMonitor(monitor, 1));
            } else {
                File srcFS = getFSFromPluginDir(importPath);
                importFilesFromFS(
                    srcFS,
                    destPath,
                    new SubProgressMonitor(monitor, 1));
            }

        } catch (CoreException e) {
            throw new InvocationTargetException(e);
        }
    }

    /**
     * @param srcDir
     * @param destPath
     * @param monitor
     * @throws InvocationTargetException
     * @throws InterruptedException
     */
    private void importFilesFromFS(
        File srcDir,
        IPath destPath,
        IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        ImportOperation op =
            new ImportOperation(
                destPath,
                srcDir,
                FileSystemStructureProvider.INSTANCE,
                fOverwriteQuery);
        op.setCreateContainerStructure(false);
        op.run(monitor);
    }

    /**
     * @param srcZipFile
     * @param destPath
     * @param monitor
     * @throws InvocationTargetException
     * @throws InterruptedException
     */
    private void importFilesFromZip(
        ZipFile srcZipFile,
        IPath destPath,
        IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException {
        ZipFileStructureProvider structureProvider =
            new ZipFileStructureProvider(srcZipFile);
        ImportOperation op =
            new ImportOperation(
                destPath,
                structureProvider.getRoot(),
                structureProvider,
                fOverwriteQuery);
        op.run(monitor);
    }

    /**
     * @param pluginRelativePath
     * @return ZipFile representing the zip file at pluginRelativePath 
     * @throws CoreException
     */
    private ZipFile getZipFileFromPluginDir(String pluginRelativePath)
        throws CoreException {
        try {
            URL starterURL =
                new URL(
                    MainPlugin.getDefault().getDescriptor().getInstallURL(),
                    pluginRelativePath);
            return new ZipFile(Platform.asLocalURL(starterURL).getFile());
        } catch (IOException e) {
            String message = pluginRelativePath + ": " + e.getMessage();
            Status status =
                new Status(
                    IStatus.ERROR,
                    MainPlugin.getPluginId(),
                    IStatus.ERROR,
                    message,
                    e);
            throw new CoreException(status);
        }
    }

    /**
     * @param pluginRelativePath
     * @return a File corresponding to the file at pluginRelativePath
     * @throws CoreException
     */
    private File getFSFromPluginDir(String pluginRelativePath)
        throws CoreException {
        URL starterURL;
        try {
            starterURL =
                new URL(
                    MainPlugin.getDefault().getDescriptor().getInstallURL(),
                    pluginRelativePath);

            return new File(Platform.asLocalURL(starterURL).getPath());
        } catch (MalformedURLException e) {
            String message = pluginRelativePath + ": " + e.getMessage();
            Status status =
                new Status(
                    IStatus.ERROR,
                    MainPlugin.getPluginId(),
                    IStatus.ERROR,
                    message,
                    e);
            throw new CoreException(status);
        } catch (IOException e) {
            String message = pluginRelativePath + ": " + e.getMessage();
            Status status =
                new Status(
                    IStatus.ERROR,
                    MainPlugin.getPluginId(),
                    IStatus.ERROR,
                    message,
                    e);
            throw new CoreException(status);
        }

    }

    /**
     * @param root
     * @param name
     * @param natureIds
     * @param referencedProjects
     * @param monitor
     * @return
     * @throws InvocationTargetException
     */
    private IProject configNewProject(
        IWorkspaceRoot root,
        String name,
        String[] natureIds,
        IProject[] referencedProjects,
        IProgressMonitor monitor)
        throws InvocationTargetException {
        try {
            IProject project = root.getProject(name);
            if (!project.exists()) {
                project.create(null);
            }
            if (!project.isOpen()) {
                project.open(null);
            }
            IProjectDescription desc = project.getDescription();
            desc.setLocation(null);
            desc.setNatureIds(natureIds);
            desc.setReferencedProjects(referencedProjects);

            project.setDescription(desc, new SubProgressMonitor(monitor, 1));

            return project;
        } catch (CoreException e) {
            throw new InvocationTargetException(e);
        }
    }

    /**
     * @return 
     */
    public IResource getElementToOpen() {
        return fElementToOpen;
    }

}
