/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import javax.swing.tree.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import org.omg.DynamicAny.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlArray represents an CORBA IDL array value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlAray object has child nodes representing the content of the IDL array it represents. The type and
 * value of the child node correspond with the type and value of the elements in the array it represents.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlArray extends IdlNode implements IdlWritable {       
    protected int size;
    protected transient TypeCode content;
    
    protected IdlArray() {
        isLeaf = false;
        isMutable = false;
        setUserObject(this);
    }
          
    protected IdlArray(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlArray(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(TypeCode tc) {
        try {
            this.tc = tc;
            if (tc.kind().value() == TCKind._tk_array) {
                id = "";
                try {
                    id = tc.id();
                    type = "array " + id;
                } catch (Exception ex) {
                    // anonymous arrays will cause an exception
                    type = "array";
                }
                size = tc.length();
                value = "" +  size;
                content = tc.content_type();
                removeAllChildren();
                for (int i = 0; i < tc.length(); i++) {
                    IdlNode m = create(content);
                    m.field = "[" + i + "]";
                    add(m); 
                }                                       
            } else if (tc.kind().value() == TCKind._tk_alias) {
                id = tc.id();
                type = id;
                TypeCode stc = tc.content_type();
                while (stc.kind().value() == TCKind._tk_alias) {
                    stc = stc.content_type();
                }
                if (stc.kind().value() != TCKind._tk_array) {
                    throw new RuntimeException("Array type expected");
                }
                size = stc.length();
                value = "" +  size;
                content = stc.content_type();
                removeAllChildren();
                for (int i = 0; i < stc.length(); i++) {
                    IdlNode m = create(content);
                    m.field = "[" + i + "]";
                    add(m); 
                }
            } else {
                throw new RuntimeException("Union type expected");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    protected void setNode(Any any) {
        try {
            setNode(any.type());
            org.omg.CORBA.portable.InputStream in = any.create_input_stream();
            removeAllChildren();
            for (int i = 0; i < size; i++) {
                Any memberAny = orb.create_any();
                memberAny.read_value(in, content);
                IdlNode m = create(memberAny);
                m.field = "[" + i + "]";
                add(m);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Return the number of array elements.
     *
     * @return The array size.
     */
    public int getSize() {
        return size;
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            any.type(tc);
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).toAny().write_value(out);
            }
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).write(os);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        try {
            for (int i = 0; i < size; i++) {
                ((IdlNode)getChildAt(i)).read(is);
            }                                    
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
    
    // XML section

    /**
     * Creates new IdlArray object from an XML string.
     * XML format example:
     *
     * <pre>
     *   &lt;array id="IDL:examples/MyArray:1.0" length=1&gt;
     *     &lt;double&gt;3.2&lt;/double&gt;
     *   &lt;/array&gt;   
     * </pre>
     *
     * @param xml The XML string from which to create a ne IdlArray instance.
     */
    public IdlArray(String xml) {
        this(XmlNode.getNode(xml));
    }

    IdlArray(Node n) {
        this(null, n);        
    }
        
    IdlArray(org.omg.CORBA.TypeCode t, Node n) {
        this();
        try {
            id = XmlNode.getId(n);
            if (!id.equals("")) {
                t = XmlNode.type(id);
            }
            setNode(t);          
            org.omg.CORBA.TypeCode content = tc.content_type();

            if (XmlNode.getLength(n) != size) {     
                throw new RuntimeException("Array length does not match actual length " + size + ".");
            }       
            Node[] nodes = XmlNode.childElements(n);
            
            if (nodes.length != size) {
                throw new RuntimeException("Number of array elements does not match " + size + ".");
            }
            removeAllChildren();
            for (int i = 0; i < size; i++) {
                IdlNode m = XmlNode.getIdlNode(content, nodes[i]);
                m.setField("[" + i + "]");
                add(m);
            }
            value = "" + size;
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlArray n, IdlWriter w) {
        w.write_start_array(n.getId(), n.getSize());
        for(int i = 0; i < n.getChildCount(); i++) {
            XmlNode.write((IdlNode)n.getChildAt(i), w);
        }        
        w.write_end_array();
    } 
}