/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.CORBA.Any;
import org.omg.DynamicAny.*;

import java.util.*;
import java.lang.reflect.*;

/**
 * The class IdlParameter represents a parameter of an operation on an CORBA IDL interface. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlParameter node has an direction attribute which has a value of "in", "inout", "out" or "return" according
 * the parameter direction it represents.
 * An IdlParameter node also has a name attribute corresponding to the name of the parameter.
 * If the IdlParameter node respresnts a return value, the name attribute is "return".
 * An IdlParameter object has one child corresponding to the parameter type.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlParameter extends IdlNode implements IdlWritable {       
    protected String name;
    protected String dir;

    protected IdlParameter() {
        isLeaf = false;
        isMutable = false;
        setUserObject(this);
        type = "parameter";
    }
    
    protected IdlParameter(String dir, String name, Any any) {
        this();
        try {
            this.dir = dir;
            this.name = name;
            IdlNode n = create(any);
            n.field = name;
            add(n);           
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }

    /**
     * Returns the parameter name.
     *
     * @return The name of the parameter.
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the parameter direction.
     *
     * Possible directions are in, inout, out and return.
     *
     * @return The direction of the parameter.
     */
    public String getDirection() {
        return dir;
    }

    /**
     * Returns a string representation of this parameter.
     *
     * @return A string representation of this parameter.
     */
    public String toString() {
        if (dir.equals("return")) {
            return dir;
        } else {        
            return dir + " " + name;
        }
    }

    /**
     * Return the IdlNode child node for this parameter.
     *
     * @return The IdlNode child node for this parameter.
     */
    public IdlNode getParameter() {
        return (IdlNode)getChildAt(0);
    }
    
    /**
     * Creates an IdlNode child parameter value from the content of the any.
     *
     * @param The any from which to create an IdlNode child.
     */
    public void setValue(Any any) {
        removeAllChildren();
        IdlNode n = create(any);
        n.field = name;
        add(n);
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        ((IdlNode)getChildAt(0)).write(os);
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        ((IdlNode)getChildAt(0)).read(is);
    }
        
    // XML section

    /**
     * Create an IdlParameter from an Xml representation.
     *
     * The XML format example
     * <pre>
     *  &lt;inout name="p2"&gt;
     *   &lt;long&gt;100&lt;/long&gt;
     *  &lt;/inout&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlParameter instance.
     */
    public IdlParameter(String xml) {
        this(XmlNode.getNode(xml));
    }
    
    IdlParameter(Node n) {
        this();
        try {
            if (n.getNodeName().toUpperCase().equals("IN")) {
                dir = "in";
                name = XmlNode.getName(n);
            } else if (n.getNodeName().toUpperCase().equals("OUT")) {
                dir = "out";
                name = XmlNode.getName(n);
            } else if (n.getNodeName().toUpperCase().equals("INOUT")) {
                dir = "inout";
                name = XmlNode.getName(n);
            } else if (n.getNodeName().toUpperCase().equals("RETURN")) {
                name = "return";
                dir = "return";
            }
            IdlNode nn = XmlNode.getIdlNode(XmlNode.firstChildElement(n));
            nn.setField(name);
            add(nn);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.toString());
        }       
    }

    IdlParameter(String dir, String name, org.omg.CORBA.TypeCode tc) {
        this();
        try {
            this.dir = dir;
            this.name = name;
            IdlNode n = create(tc);
            n.setField(name);
            add(n);
         } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.toString());
        }
    }
                
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        write(this, w);
    }

    public static void write(IdlParameter n, IdlWriter w) {
        if (n.getDirection().equals("in")) {
            w.write_start_inpar(n.getName());
            XmlNode.write((IdlNode)n.getParameter(), w);
            w.write_end_inpar();
        } else if (n.getDirection().equals("out")) {
            w.write_start_outpar(n.getName());
            XmlNode.write((IdlNode)n.getParameter(), w);
            w.write_end_outpar();
        } else if (n.getDirection().equals("inout")) {
            w.write_start_inoutpar(n.getName());
            XmlNode.write((IdlNode)n.getParameter(), w);
            w.write_end_inoutpar();   
        } else if (n.getDirection().equals("return")) {
            w.write_start_return();
            XmlNode.write((IdlNode)n.getParameter(), w);
            w.write_end_return();
         }
    }   
}