package org.coach.tracing.server.viewer;

import java.awt.print.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.coach.tracing.server.EventRecord;
import org.coach.tracing.server.EventDataBase;
import org.coach.tracing.api.*; // from tracing.idl

public class Diagram extends JPanel implements Printable {
    private TraceViewerFrame frame;
    private static Set trailLabels = new HashSet();
    
    //the space between the first message and the top of the diagram
    public final static int lead_top = 10;

    //the space between the last message and the bottom of the diagram
    public final static int lead_bottom = 100;

    //the space between two neighbour messages
    public final static int space_between = 20;
    private static ScaleManager sm;    
    private static LinkedList eventList = new LinkedList();
    private static HashMap eventMap = new HashMap();
    private static HashMap eventGuiMap = new HashMap();
    private static HashMap messageGuiMap = new HashMap();
    private static HashMap identityGuiMap = new HashMap();
    private static EventDataBase eventDB;
    private HashMap colorMap = new HashMap();
    private String causalityTrailId;
    private long currentIndex = 0;
    private long range = 0;
    private EventGui eventGuiPool;
    private TrailLabelData[] trailLabelData;
    private long eventCount = 0;
    private boolean threadMode = false;

    public Diagram(TraceViewerFrame f) {
        sm = ScaleManager.getScaleManager();
        frame = f;
        setDoubleBuffered(true);
        setPreferredSize(new Dimension(0, 0));
        setMinimumSize(new Dimension(0, 0));
        setLayout(null);
        eventGuiPool = new EventGui(this);
    }
    
    void setEventDB(EventDataBase db) {
        clear();
        eventDB = db;
        updateStatus(0);
        repaint();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
        return 0;
    }
        
    public void setRange(long range) {
        this.range = range;
        paintEvents(currentIndex);
    }
    
    public void setThreadViewMode(boolean mode) {
        threadMode = mode;
        repaintEvents();
    }
            
    public static ScaleManager getScaleManager() {
        return sm;
    }
    
    private void addMessage(EventRecord mm) {
        EventGui e1 = (EventGui)eventGuiMap.get(mm.key());
        EventGui e2 = (EventGui)eventGuiMap.get(mm.getLinkKey());

        if (threadMode) {
            
            ThreadGui threadGui = (ThreadGui)identityGuiMap.get(ThreadNode.get(IdentityNode.getProcessId(mm.getIdentityKey()), mm.getThreadId()).getVisible());
            if(e1 != null && e2 != null) {
                MessageGui mmg = new MessageGui(this, e1, e2, threadGui.getWidth());
                messageGuiMap.put(mm.key(), mmg);
                add(mmg, null, 0);
            }
        } else {            
            IdentityGui identityGui = (IdentityGui)identityGuiMap.get(IdentityNode.get(mm.getIdentityKey()).getVisible());
            if(e1 != null && e2 != null) {
                MessageGui mmg = new MessageGui(this, e1, e2, identityGui.getWidth());
                messageGuiMap.put(mm.key(), mmg);
                add(mmg, null, 0);
            }
        }
    }

    private void addEvent(EventRecord r, int position) {        
        if (threadMode) {
            String processId = IdentityNode.getProcessId(r.getIdentityKey());
            ThreadNode v = ThreadNode.get(processId, r.getThreadId()).getVisible();
            if (v != null) {
                // This event is on a visible thread.
                ThreadGui threadGui = (ThreadGui)identityGuiMap.get(v);
                if (threadGui == null) {
                    throw new RuntimeException("ThreadGui is missing!");
                }
                        
                EventGui eg = eventGuiPool.getEventGui(r);
                eventGuiMap.put(r.key(), eg);
                eg.setPosition(position);
                int x = threadGui.getXoffset() - (eg.getWidth() / 2);
                int y = (int)((lead_top + (space_between * position)) * sm.getScale());
                eg.setLocation(x, y);
                add(eg, null, 0);
            }
        } else {
            IdentityNode v = IdentityNode.get(r.getIdentityKey()).getVisible();
            if (v != null) {
                // This event is on a visible identity.
                IdentityGui identityGui = (IdentityGui)identityGuiMap.get(v);
                if (identityGui == null) {
                    throw new RuntimeException("IdentityGui is missing!");
                }
                        
                EventGui eg = eventGuiPool.getEventGui(r);
                eventGuiMap.put(r.key(), eg);
                eg.setPosition(position);
                int x = identityGui.getXoffset() - (eg.getWidth() / 2);
                int y = (int)((lead_top + (space_between * position)) * sm.getScale());
                eg.setLocation(x, y);
                add(eg, null, 0);
            }
        }                
        if (!r.getTrailLabel().equals("")) {
            trailLabels.add(r.getTrailLabel());
        }
    }
    
    public synchronized void clear() {
        removeAll();
        eventGuiMap.clear();
        messageGuiMap.clear();
        colorMap.clear();
        identityGuiMap.clear();
        currentIndex = 0;
        range = 0;
        eventCount = 0;
    }

    public void print(Graphics g) {
        Component clist[] = getComponents();
        int ncomponents = clist.length;
        Rectangle clip = g.getClipBounds();
        for (int i = ncomponents - 1; i >= 0; i--) {
            Component comp = clist[i];
            if (comp != null) {
                Rectangle cr = comp.getBounds();
                if ((clip == null) || cr.intersects(clip)) {
                    Graphics cg = g.create(cr.x, cr.y, cr.width, cr.height);
                    cg.setFont(comp.getFont());
                    try {
                        comp.paint(cg);
                    } finally {
                        cg.dispose();
                    }
                }
            }
        }
    }
    
    public void createEventBrowser(Long key, Point p) {
        try {
            frame.createEventBrowser((EventRecord)eventMap.get(key), p);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public EventGui getEventGui(Long key) {
        return (EventGui)eventGuiMap.get(key);
    }

    public MessageGui getMessageGui(Long key) {
        return (MessageGui)messageGuiMap.get(key);
    }

    public synchronized void colour_causality_chain(Long key) {
        try {
            EventRecord r = (EventRecord)eventMap.get(key);
            causalityTrailId = r.getTrailId();
            
            Iterator it = eventList.iterator();
            while(it.hasNext()) {
                r = (EventRecord)it.next();
                if (r.getTrailId().equals(causalityTrailId)) {
                    //colour the event
                    colorMap.put(r.key(), java.awt.Color.magenta);
                }
            }                
            repaintEvents();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void uncolour() {
        colorMap.clear();
        causalityTrailId = null;
        if (trailLabelData != null) {
            for (int i = 0; i < trailLabelData.length; i++) {
                trailLabelData[i].active = false;
            }
        }
        repaint();
    }
    
    public java.awt.Color getColor(Object obj) {
        return (java.awt.Color)colorMap.get(obj);
    }

    public void setTrailLabelData(TrailLabelData[] trailLabelData) {
        this.trailLabelData = trailLabelData;
    }
    
    public TrailLabelData[] getTrailLabelData() {
        return trailLabelData;
    }

    public String[] getTrailLabels() {
        String[] labels = new String[trailLabels.size()];
        trailLabels.toArray(labels);
        return labels;
    }
                
    public void colorTrail() {
        try {
            if (trailLabelData != null) {
                Iterator it = eventGuiMap.keySet().iterator();
                while(it.hasNext()) {
                    Long key = (Long)it.next();
                    EventRecord r = (EventRecord)eventMap.get(key);
                    for (int i = 0; i < trailLabelData.length; i++) {
                        if (r.getTrailLabel().equals(trailLabelData[i].name) && trailLabelData[i].active) {
                            colorMap.put(r.key(), trailLabelData[i].color);
                        }
                    }
                }
            }
            if (causalityTrailId != null) {
                Iterator it = eventList.iterator();
                while(it.hasNext()) {
                    EventRecord r = (EventRecord)it.next();
                    if (r.getTrailId().equals(causalityTrailId)) {
                        //colour the event
                        colorMap.put(r.key(), java.awt.Color.magenta);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void updateStatus(long events) {
        eventCount = events;
        if (frame != null) {
            long messages = eventDB.getMessageCount();
            long identities = eventDB.getIdentityCount();
            frame.updateStatusBar(eventCount, identities, messages);
        }
    }

    /**
     * Clears the display and repaints the events starting with the given index
     */
    public void repaintEvents() {
        paintEvents(currentIndex);
    }
    
    public void paintEvents(long startIndex) {
        try {
            if (eventCount == 0) {
                // No event has been received yet.
                return;
            } 
            currentIndex = startIndex;
            long endIndex = Math.min(startIndex + range, eventCount);      
    
            Iterator it = eventGuiMap.values().iterator();
            while(it.hasNext()) {
                EventGui eg = (EventGui)it.next();
                eventGuiPool.releaseEventGui(eg);
            }
            eventList.clear();
            eventMap.clear();
            eventGuiMap.clear();
            identityGuiMap.clear();
            messageGuiMap.clear();  
            IdentityGui.reset();
            ThreadGui.reset(); 
            removeAll();                              

            EventRecord[] events = eventDB.getEvents(startIndex, endIndex);
            for (int i = 0; i < events.length; i++)
            {
                IdentityNode.get(events[i].getIdentityKey());
                String processId = IdentityNode.getProcessId(events[i].getIdentityKey());
                ThreadNode.get(processId, events[i].getThreadId());
                eventMap.put(events[i].key(), events[i]);
                eventList.add(events[i]);
            }
            

            // Now create identitylines for all visible identities.
            int width = 0;
            if (threadMode) {
                ThreadNode[] v = ThreadNode.getVisibles();
                for (int i = 0; i < v.length; i++) {
                    ThreadGui threadGui = (ThreadGui)identityGuiMap.get(v[i]);
                    if (threadGui == null) {
                        threadGui = new ThreadGui(this, v[i].getName(), v[i]);
                        identityGuiMap.put(v[i], threadGui);
                        add(threadGui);
                        width += threadGui.getWidth();
                    }
                }                                   
            } else {
                IdentityNode[] v = IdentityNode.getVisibles();
                for (int i = 0; i < v.length; i++) {
                    IdentityGui identityGui = (IdentityGui)identityGuiMap.get(v[i]);
                    if (identityGui == null) {
                        identityGui = new IdentityGui(this, v[i].getName(), v[i].getType(), v[i]);
                        identityGuiMap.put(v[i], identityGui);
                        add(identityGui);
                        width += identityGui.getWidth();
                    }
                }
            }
            
            // set the new preferred size
            setSize(new Dimension(getHeight(), width));
            setPreferredSize(new Dimension(getHeight(), width));
                            
            // Add the event to the display              
            for(int index = 0; index < eventList.size(); index++) {
                EventRecord r = (EventRecord)eventList.get(index);
                
                // Could add a filter here to decide if the event is visible and must be
                // added. The event position can then be compressed to allow additional events to
                // be displayed.

                addEvent(r, (int)(r.getCurrentIndex() - startIndex));
            }

            // Add the messages to the display
            for(int index = 0; index < eventList.size(); index++) {
                EventRecord r = (EventRecord)eventList.get(index);
                
                if (r.isLinked() && eventGuiMap.get(r.getLinkKey()) != null && getMessageGui(r.key()) == null && getMessageGui(r.getLinkKey()) == null) {
                    addMessage(r);
                }
            }
                                                
            colorTrail();
            
            repaint();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
