/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Areski Flissi.

====================================================================*/

package org.objectweb.comanche.lib;

// Package dependencies.
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.LineNumberReader;
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.net.Socket;

import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.api.OptionArgument;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;

/**
 * Comanche is a micro HTTP server.
 */

public class Application
     extends ApplicationBase
{ 
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The command line argument for the root directory. */
    private OptionArgument rootDirectoryArgument_;

    /** Th command line argument for the TCP/IP port. */
    private OptionArgument portArgument_;
    
    /** Th command line argument for the InetAddress group. */
	private OptionArgument groupAddress_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    Application()
    {
        // Call the ApplicationBase constructor.
        super(
            // Init the command line.
            new DefaultCommandLine("comanche",
                                   "",
                                   "Run the OpenCCM Comanche HTTP server.",
                                   true)
        );

        // Set zero argument to the command line.
        getCommandLine().setArguments(new String[0]);

        // Add the command line argument for the root directory.
        rootDirectoryArgument_ =
            new DefaultOptionArgument("--rootdir",
                                      "directory",
                                      "Set the root directory, default is ./",
                                      "."
            );
        getCommandLine().addOption(rootDirectoryArgument_);

        // Add the command line argument for the TCP/IP port.
        portArgument_ =
            new DefaultOptionArgument("--port",
                                      "port",
                                      "Set the TCP/IP port, default is 8080.",
                                      "8080"
            );
        getCommandLine().addOption(portArgument_);
        
        // Add the command line argument for the groupAddress argument.
		groupAddress_ =
			new DefaultOptionArgument(
				"--group",
				"groupAddress",
				"Set the address group, default is 224.0.0.100.",
				"224.0.0.100");
		getCommandLine().addOption(groupAddress_);
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Starts the main function.
     *
     * Must be defined into subclasses.
     *
     * @param args The command line arguments.
     *
     * @return The status of the application.
     */
    public int
    start(String[] args)
    {
        // Obtain the root directory for the HTTP and Multicast server.
        String rootDirectory = rootDirectoryArgument_.getArgument();

		// Starting the Multicast Server
		String addgroup = groupAddress_.getArgument();		
		MulticastServer mcastserver = new MulticastServer(addgroup,rootDirectory);
		mcastserver.start();
		System.err.println("The OpenCCM Comanche Server is waiting Multicast requests on mcast://" + addgroup);


        // Obtain the TCP/IP port for the HTTP server.
        int port = 0;
        try
	{
            port = Integer.parseInt(portArgument_.getArgument());
        } catch(NumberFormatException ex) {
            report_exception(ex);
            return -1;
        }

        // Create the TCP/IP socket for the HTTP server.
        ServerSocket server = null; 
        try
	{
            server = new ServerSocket(port); 
        } catch(IOException ex) {
            report_exception(ex);
            return -1;
        }

        // Obtain the URL to access the HTTP server.
        String url = null;
        try
	{
            url = "http://" + InetAddress.getLocalHost().getHostName()
                      + ':' + server.getLocalPort();
        } catch(UnknownHostException ex) {
            report_exception(ex);
            return -1;
        }

        System.err.println("The OpenCCM Comanche Server is waiting HTTP requests on " + url);

        // Infinite loop of the HTTP server.
        while (true)
        {
            try
	    {
                getConsole().message("Waiting requests on " + url);
                // Accept a new client.
                Socket client = server.accept();

                InputStreamReader in = new InputStreamReader(client.getInputStream()); 
                PrintStream out = new PrintStream(client.getOutputStream());

                // Read the client request.  
                String request = new LineNumberReader(in).readLine(); 
                getConsole().message(request);

                if (request.startsWith("GET "))
                {
                    File f = new File(rootDirectory + request.substring(4, request.indexOf(' ', 4)));
                    if (f.exists() && !f.isDirectory())
                    { 
                        InputStream is = new FileInputStream(f); 
                        byte[] data = new byte[is.available()]; 
                        is.read(data); 
                        is.close(); 
                        out.print("HTTP/1.0 200 OK\n\n"); 
                        out.write(data); 
                    } else { 
                        out.print("HTTP/1.0 404 Not Found\n\n"); 
                        out.print("<html>Document not found.</html>"); 
                    } 
                }
                out.close(); 
                client.close();
            } catch(Exception ex) {
                report_exception(ex);
            }
        }
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {    	
    	Application application = new Application();    		
        application.runMain(args);             
    }
}
