/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.lib;

// Package dependencies.
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;
import org.objectweb.corba.ast.api.StringList;

/**
 * SegmentDeclImpl is a wrapper class for
 * CIDL segment declarations.
 *
 *
 * Inherits from:
 *
 * - DeclarationImpl as they are IDL declarations,
 *
 * - SegmentDecl: OMG IDL for SegmentDecl.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.3
 */

public class SegmentDeclImpl
     extends DeclarationImpl
  implements org.objectweb.corba.ast.api.SegmentDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated abstract storagehome. */
    private AbstractStorageHomeDeclImpl ash_;

    /** The list of facets. */
    private StringListImpl facets_;

    /** The list of sinks. */
    private StringListImpl sinks_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    SegmentDeclImpl(Repository rep,
                    ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        ash_ = null;
        facets_ = new StringListImpl();
        sinks_ = new StringListImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Methods for the inherited DeclarationImpl class.
    //
    // ==================================================================

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the any declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     *       as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        java.util.Vector seg_depend = new java.util.Vector();
        Declaration[] depend = null;

        // Segment persistence declaration
        if (getAbstractStorageHome() != null)
        {
            seg_depend.add( getAbstractStorageHome() );
            depend = getAbstractStorageHome().getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ( (depend[j] != this) &&
                     (seg_depend.indexOf(depend[j]) == -1) )
                    seg_depend.add(depend[j]);
            }
        }

        // Facets / Sinks list
        /* No dependencies because facets and sinks are dependencies
           of the component executor (via the component). */

        dependencies_ = (Declaration[])seg_depend.toArray(new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * This method is implemented into DeclarationImpl subclasses.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
      return DeclarationKind.dk_segment;
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.corba.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.corba.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.corba.ast.api.SegmentDecl
    //
    // ==================================================================

    /**
     * Set the associated abstract storagehome.
     *
     * @param ash - The associated abstract storagehome.
     */
    public void
    setAbstractStorageHome(AbstractStorageHomeDecl ash)
    {
        ash_ = (AbstractStorageHomeDeclImpl)ash;
    }

    /**
     * Obtain the associated abstract storagehome.
     *
     * @return The associated abstract storagehome.
     */
    public AbstractStorageHomeDecl
    getAbstractStorageHome()
    {
        return ash_;
    }

    /**
     * Obtain the list of implemented facets.
     *
     * @return The list of facets.
     */
    public StringList
    getFacetList()
    {
        return facets_;
    }

    /**
     * Obtain the list of implemented sinks.
     *
     * @return The list of sinks.
     */
    public StringList
    getSinkList()
    {
        return sinks_;
    }
}
