/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.cmdline.lib.DefaultOptionArguments;
import org.objectweb.corba.ast.api.FileScope;
import java.util.Vector;
import java.util.ArrayList;

/**
 * Implementation of the cidl_cif command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.2
 */

public class CIDLtoCIF
     extends CompilerGeneratorBase
  implements org.objectweb.corba.command.api.CIDLtoCIF
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Include Options. */
    private OptionMultipleArguments userInclude_;

    /** Output directory Option for CIF skeletons. */
    org.objectweb.util.cmdline.api.OptionArgument odir_; // -d <directory>

    /** Output directory Option for CIDL dependencies. */
    org.objectweb.util.cmdline.api.OptionArgument depdir_; // -dep <directory>

    /** Unwanted prefix for CIDL dependencies. */
    org.objectweb.util.cmdline.api.OptionArguments noprefix_; // --noprefix <prefix>

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    CIDLtoCIF()
    {
        // Calls the CompilerGeneratorBase constructor.
        super(new DefaultCommandLine("cidl_cif",
                                     "file",
                                     new String[] {
                                        "Compile an OMG CIDL file and generate its associated:",
                                        "- local CIF interfaces",
                                        "- Java component and home executor skeletons",
                                        "- class dependencies files"
                                     },
                                     true),
              "generated_cif.idl");

        // Inits internal state.
        userInclude_ = new OptionMultipleArguments(
                               "-ipath",
                               new String[] { "-ipath" },
                               "file",
                               new String[] {
                                   "Add a #include \"file\" statement where",
                                   "file must be the complete file path"
                               },
                               "",
                               true);
        getCommandLine().addOption(userInclude_);

        // Specify CIF skeletons output base directory
        odir_ = new DefaultOptionArgument(
                        new String[] { "-d" },
                        "outputdir",
                        new String[] {
                          "Generate Java implementations into directory <outputdir>,",
                          "default is generated/"
                        },
                        "generated" );
        getCommandLine().addOption(odir_);

        // Specify CIDL dependencies output base directory
        depdir_ = new DefaultOptionArgument(
                        new String[] { "-dep" },
                        "dependencies_dir",
                        new String[] {
                          "Generate CIDL class dependencies into directory <dependencies_dir>,",
                          "default is dependencies/"
                        },
                        "dependencies" );
        getCommandLine().addOption(depdir_);

        // Specify unwanted prefix
        noprefix_ = new DefaultOptionArguments(
                        new String[] { "--noprefix" },
                        "aprefix",
                        new String[] {
                          "CIDL class dependencies starting with <aprefix>",
                          "will not be included."
                        } );
        getCommandLine().addOption(noprefix_);
   }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];

        // Inits the OpenCCM Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Compiles the file.
        FileScope fileScope = compile(filename);
        if(fileScope == null)
            return -1;

        // Generates CIF
        if (!generate_cif(fileScope,
                          filename, // Input File
                          getOutputFileName(), // IDL File
                          getCIFOutputDirectory(), // Base directory for Java Mapping
                          userInclude_.getOptionArguments()))
            return -1;

        // Generates CIDL dependencies
        if ( !generate_cif_dependencies(fileScope,
                                        getDependenciesOutputDirectory(),
                                        getNoPrefix()) )
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.CIDLtoCIF
    //
    // ==================================================================

    /**
     * Obtains CIF skeletons output directory.
     *
     * @result The CIF skeletons output directory.
     */
    public String
    getCIFOutputDirectory()
    {
        return odir_.getArgument();
    }

    /**
     * Obtains CIDL dependencies output directory.
     *
     * @result The output directory.
     */
    public String
    getDependenciesOutputDirectory()
    {
        return depdir_.getArgument();
    }

    /**
     * Obtains unwanted prefixs for CIDL dependencies.
     *
     * @result The list of prefixs.
     */
    public String[]
    getNoPrefix()
    {
        return noprefix_.getOptionValues();
    }

    /**
     * Generates CIF interfaces and skeletons
     * from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile - The source file.
     * @param outputfile - The file name for the IDL mapping.
     * @param base_dir - The base directory for java mapping.
     * @param userIncludes - User files to include "...".
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_cif(FileScope filescope,
                 String inputfile,
                 String outputfile,
                 String base_dir,
                 String[] userIncludes)
    {
        Vector vect = null;

         // Generates CIF interfaces in the AST.
        org.objectweb.corba.generator.cif_idl.api.CIF_IDLGenerator cif_gen =
            new org.objectweb.corba.generator.cif_idl.lib.CIF_IDLGenerator( getAST(),
                                                                            getComponentRepository() );
        try
        {
            vect = cif_gen.declare_cif(filescope);
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }

        // Switch the OpenCCM Interface Repository in mode OMG IDL 2.x.
        getComponentRepository().as_IDL2_repository();

        // Launchs the IDL2 generator.
        org.objectweb.corba.generator.idl.api.IDL2Generator idl2_gen =
            new org.objectweb.corba.generator.idl.lib.IDL2Generator( getAST() );
        idl2_gen.initialize(outputfile,
                            outputfile,
                            new ArrayList(), //includes
                            new ArrayList(java.util.Arrays.asList(userIncludes)), //user includes
                            null,
                            "CIDLtoCIF" );

        // Genarates CIF interfaces java mapping.
        getConsole().message("Generating CIF mapping from " + inputfile + "...");
        try
        {
            idl2_gen.generate(vect);
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        getConsole().message("OMG IDL mapping generated in " + outputfile + ".");

        // Generates CIF skeletons
        org.objectweb.corba.generator.cif.lib.CIF_JavaGenerator cif_impl_gen =
            new org.objectweb.corba.generator.cif.lib.CIF_JavaGenerator( getAST() );
        getConsole().message("Generating CIF implementations...");
        try
        {
            cif_impl_gen.cif_to_java(base_dir, filescope);
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        getConsole().message("CIF implementations generated.");

        // Deletes CIF modules in I.R.
        cif_gen.remove_decls();

        // Switch the OpenCCM Interface Repository in mode OMG IDL 3.x.
        getComponentRepository().as_IDL3_repository();

        return true;
    }

    /**
     * Generates CIF dependencies from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope - The source AST FileScope returned by the compile() method.
     * @param base_dir - The base directory for dependencies output.
     * @param unwanted_prefixs - Unwanted prefixs for CIDL dependencies.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_cif_dependencies(FileScope filescope,
                              String base_dir,
                              String[] unwanted_prefixs)
    {
        org.objectweb.corba.generator.cidl.api.CIDLFileDependenciesResolver dep_resolver = null;

        dep_resolver =
            new org.objectweb.corba.generator.cidl.lib.CIDLFileDependenciesResolver( getAST() );
        getConsole().message("Generating CIDL dependencies...");
        try
        {
            dep_resolver.set_filters(unwanted_prefixs);
            dep_resolver.generate_dependencies( filescope, base_dir );
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        getConsole().message("CIDL dependencies generated.");

        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        CIDLtoCIF cidl2cif = new CIDLtoCIF();
        cidl2cif.runMain(args);
    }
}
