/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.corba.ast.api.FileScope;

/**
 * This class allows you to generate Java mapping from a PSDL file.
 *
 * @author <a href="mailto:Chrsitophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PSDLtoJava
     extends CompilerGeneratorBase
  implements org.objectweb.corba.command.api.PSDLtoJava
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Output directory Option. */
    org.objectweb.util.cmdline.api.OptionArgument odir_; // -d <directory>

    /** User JDO properties file name. */
    org.objectweb.util.cmdline.api.OptionArgument jdo_props_; // -jdoprop <file_name>

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public
    PSDLtoJava()
    {
        // Calls the CompilerGeneratorBase constructor.
        super(new DefaultCommandLine("psdl_java",
                                     "file",
                                     "Compile an OMG PSDL file and generate its associated Java mapping",
                                     true));

        // Specify output base directory
        odir_ = new DefaultOptionArgument(
                        new String[] { "-d" },
                        "outputdir",
                        new String[] {
                          "Generate Java mapping into directory <outputdir>,",
                          "default is generated/"
                        },
                        "generated" );
        getCommandLine().addOption(odir_);

        // Specify User JDO properties file name
        jdo_props_ = new DefaultOptionArgument(
                        new String[] { "-jdoprop" },
                        "jdo_properties_file",
                        new String[] {
                          "Get user classes definition from the properties file <jdo_properties_file>",
                          "default is \"jdouser.properties\""
                        },
                        "jdouser.properties" );
        getCommandLine().addOption(jdo_props_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];

        // Inits the OpenCCM Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Compiles the file.
        FileScope fileScope = compile(filename);
        if(fileScope == null)
            return -1;

        // Generates PSDL Mapping
        if (!generate_java_mapping(fileScope,
                                   filename, // Input File
                                   getOutputDirectory(), // Base directory for Java Mapping
                                   getUserJdoPropertiesFile())) // User JDO properties file
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.command.api.PSDLtoJava
    //
    // ==================================================================

    /**
     * Obtains the output directory.
     *
     * @result The output directory.
     */
    public String
    getOutputDirectory()
    {
        return odir_.getArgument();
    }

    /**
     * Obtains the user jdo properties file name.
     *
     * @result The user jdo properties file name.
     */
    public String
    getUserJdoPropertiesFile()
    {
        return jdo_props_.getArgument();
    }

    /**
     * Generates PSDL Mapping from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile - The source file.
     * @param base_dir - The base directory for java mapping.
     * @param user_properties_file - The user jdo properties file.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_java_mapping(org.objectweb.corba.ast.api.FileScope filescope,
                          String inputfile,
                          String base_dir,
                          String user_properties_file)
    {
        // Launchs the CIF generator.
        org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator gen =
            new org.objectweb.corba.generator.psdl.lib.PSDL2JavaGenerator(getAST());

        // Generates Java Mapping.
        getConsole().message("Generating Java mapping from " + inputfile + " ...");
        try
        {
            gen.psdl_to_java(base_dir, filescope, user_properties_file);
        }
        catch(org.objectweb.corba.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        getConsole().message("Java mapping generated.");

        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        PSDLtoJava psdl2java = new PSDLtoJava();
        psdl2java.runMain(args);
    }
}
