/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.popup;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.api.MenuFactory;
import org.objectweb.util.browser.api.MenuFactoryConfiguration;
import org.objectweb.util.browser.api.BrowserProperty;
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.objectweb.util.browser.gui.common.DefaultPropertyContainer;
import org.objectweb.util.browser.gui.common.ClassesInheritance;

/** The Java API's imports */
import java.util.Vector;
import java.util.List;
import java.util.Iterator;

/**
 *
 * It contains a cache memory which allow to have only one factory per class type
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class MenuPropertyContainer extends DefaultPropertyContainer {

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default contstructor
     * @param loader The object to use to load new configuration
     */
    public MenuPropertyContainer(BrowserProperty loader) {
        super(loader);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tries to load an element for the associated class name.
     * @param className The name of the class 
     * @return The founded entry or null
     */
    protected Entry getElement(String className) {
        return loader_.getMenu(className);
    }

    /**
     * Adds the menuFactory to the container.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The menuFactory to add
     * 
     */
    protected void addEntry(String id, MenuFactory object) {
        super.addEntry(id, object);
    }

    // ==================================================================
    //
    // Surcharging public methods for DefaultContextContainer class.
    //
    // ==================================================================

    /**
     * Finds the object associated value. 
     * If the key doesn't exist, it looks for in interfaces implemented 
     * by this class and in superclass of the entity
     * In order to have a string representation of the object, 
     * it takes his name (object.getClass().getName())
     * @param object The research key
     * @return The associated value - null is returned if no value exists
     */
    public Entry getRecursiveProperty(Object object) {
        Class c = object.getClass();
        // Looks for association into the cache
        if (cache_.containsKey(c.getName())) {
            return getOWLocalEntry((String) cache_.get(c.getName()));
        }
        // Gets the list of inherit classes
        ClassesInheritance ci = new ClassesInheritance(c);
        List list = ci.getInheritClasses();
        Iterator it = list.iterator();
        // To put the list of factory
        Vector inheritTypeMenu = new ContextVector();
        Vector inheritTreeMenu = new ContextVector();
        boolean isFirst = true;
        MenuFactoryConfiguration menuFactory = null;
        Entry currentMenuEntry = null;
        Entry entry = null;
        while (it.hasNext()) {
            Class class_ = (Class) it.next();
            entry = getOWLocalEntry(class_.getName());
            if (entry == null) {
                entry = tryToLoad(class_.getName());
            }
            if (entry != null) {
                if (isFirst) {
                    isFirst = false;
                    cache_.put(c.getName(), class_.getName());
                    menuFactory = (MenuFactoryConfiguration) entry.getOWValue();
                    menuFactory.setClassName(class_.getName());
                    currentMenuEntry = entry;
                } else {
                    MenuFactoryConfiguration mfc = (MenuFactoryConfiguration) entry.getOWValue();
                    inheritTypeMenu.add(mfc.getItems());
                    inheritTreeMenu.add(mfc.getChildItems());
                }
            }

        }
        if (menuFactory != null) {
            menuFactory.setInheritTypeMenu((Context[]) inheritTypeMenu.toArray(new Context[0]));
            menuFactory.setInheritTreeMenu((Context[]) inheritTreeMenu.toArray(new Context[0]));
        }
        return currentMenuEntry;
    }

    /**
     * Adds the object to the container.
     *
     * This method does not modify the given name and the target context.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The object to add
     * 
     */
    public void addEntry(String id, Object object) {
        if (object != null) {
            addEntry(id, (MenuFactory) object);
        }
    }

    /**
     * Defines a subclass of Vector which adds a context only if the given one is not empty.
     */
    protected class ContextVector extends Vector {

        /**
         * Indicates if a context is empty
         * @return true if the given context is empty
         */
        protected boolean isEmpty(Context context) {
            if (context != null) {
                EntryIterator it = context.getOWEntries();
                while (it.hasNext())
                    return false;
            }
            return true;
        }

        /**
         * Appends the given context to the end of this Vector only if it's not empty.
         */
        public boolean add(Context c) {
            if (!isEmpty(c))
                return super.add(c);
            return false;
        }

        /**
         * Appends the specified element to the end of this Vector.
         */
        public boolean add(Object o) {
            if (o != null)
                return add((Context) o);
            return false;
        }

    }

}
