/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor;

// SAX imports
import org.xml.sax.InputSource;

// importing CCM DTDs beans
import org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBeanImpl;
import org.objectweb.ccm.descriptor.corbacomponent.beans.CorbacomponentBeanImpl;
import org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBeanImpl;
import org.objectweb.ccm.descriptor.properties.beans.PropertiesBeanImpl;

public class EntityResolver implements org.xml.sax.EntityResolver {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** 
     * CORBA Software Descriptor Formal Public Identifier.
     * This identifier is "-//OMG//DTD CORBA Software Descriptor 3.0//EN". 
     */
    public static final String CSD_FPI_3_0 = "-//OMG//DTD CORBA Software Descriptor 3.0//EN";

    /** 
     * CORBA Component Descriptor Formal Public Identifier.
     * This identifier is "-//OMG//DTD CORBA Component Descriptor 3.0//EN". 
     */
    public static final String CCD_FPI_3_0 = "-//OMG//DTD CORBA Component Descriptor 3.0//EN";

    /** 
     * Component Assembly Descriptor Formal Public Identifier.
     * This identifier is "-//OMG//DTD Component Assembly Descriptor 3.0//EN". 
     */
    public static final String CAD_FPI_3_0 = "-//OMG//DTD Component Assembly Descriptor 3.0//EN";

    /** 
     * Component Property File Formal Public Identifier.
     * This identifier is "-//OMG//DTD Component Property File 3.0//EN". 
     */
    public static final String CPF_FPI_3_0 = "-//OMG//DTD Component Property File 3.0//EN";
    
    /** The CCM entity resolver singleton. */
    private static EntityResolver resolver_singleton=null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Returns a character stream reader on a file loaded in memory by
     * the current context class loader.
     *
     * @param name The name of the file to read.
     *
     * @return The character stream reader on the given file. 
     */
    protected java.io.Reader getReaderInMem(String name) {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        return new java.io.InputStreamReader(loader.getResourceAsStream(name));
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Resolve CCM DTDs based on CCM FPI described by {@link
     * #CAD_FPI_3_0}, {@link #CSD_FPI_3_0}, {@link #CCD_FPI_3_0},
     * {@link #CPF_FPI_3_0}. The implementation try to resolve
     * "properties.dtd", "corbacomponent.dtd", "softpkg.dtd",
     * "componentassembly.dtd" in the jars loaded by the context class
     * loader.
     *
     * @param publicId The public identifier. Supported identifiers
     * are {@link #CAD_FPI_3_0}, {@link #CSD_FPI_3_0}, {@link
     * #CCD_FPI_3_0} and {@link #CPF_FPI_3_0}.
     * @param systemId An alternative system identifier for dtds.
     *
     * @return An input source on the pointed DTD.
     */
    public InputSource resolveEntity (String publicId, String systemId)
    {
        java.io.Reader reader = null;
        if (publicId != null) {
            if (publicId.equals(CPF_FPI_3_0)) {
                reader = getReaderInMem("properties.dtd");
            } else if (publicId.equals(CCD_FPI_3_0)) {
                reader = getReaderInMem("corbacomponent.dtd");
            } else if (publicId.equals(CSD_FPI_3_0)) {
                reader = getReaderInMem("softpkg.dtd");
            } else if (publicId.equals(CAD_FPI_3_0)) {
                reader = getReaderInMem("componentassembly.dtd");
            }
        } 
        if (systemId != null && reader == null) {
            return new InputSource(systemId);
        }
        if (reader == null) return null;
        return new InputSource(reader);
    }

    /**
     * Register an EntityResolver for all CCM DTDs root element bean. 
     */
    public static void setCCMResolver() {
        if (resolver_singleton == null) {
            resolver_singleton = new EntityResolver();
            ComponentassemblyBeanImpl.setEntityResolver(resolver_singleton);
            CorbacomponentBeanImpl.setEntityResolver(resolver_singleton);
            SoftpkgBeanImpl.setEntityResolver(resolver_singleton);
            PropertiesBeanImpl.setEntityResolver(resolver_singleton);
        }
    }
}
