/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): Christophe Demarey.

--------------------------------------------------------------------
$Id: RepositoryZeus.java,v 1.7 2004/03/17 10:08:25 demarey Exp $
====================================================================*/

package org.objectweb.util.launcher.parser;

import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.objectweb.util.launcher.LauncherException;
import org.objectweb.util.trace.TraceSystem;

/**
 * Specialization of the repository for considering Zeus descriptions.
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class RepositoryZeus 
  implements Repository
{

    /** java properties are identified by separator value in XML files */
    public static final String separator = "@" ;

    /**
     * List of Zeus definitions.
     */
    protected HashMap descriptions_ ;
    
    /**
     * Default constructor.
     */
    public RepositoryZeus() {
        this.descriptions_ = new HashMap();
    }
    
    /**
     * Description accessor.
     * @return The list of descriptions.
     */
    protected HashMap getDescriptions() {
        return this.descriptions_;
    }
    
    /**
     * Add a new description in the repository.
     * @param identifier the identifier of the description.
     * @param description the description (ContextDescription or RunDescription) to add.
     */
    public void addDescription(String identifier, Object description) {
        TraceSystem.get("repository").debug("Registering "+identifier+" with value "+description);
        getDescriptions().put(identifier, description);
    }

    /**
     * Replace tags @NAME@ in a string.<BR>
     * NAME must be a jave property.
     *
     * @param tagged - The string to replace.
     *
     * @return A new String with all tag replacements.
     */
    protected String replaceTag(String tagged) {
        StringBuffer result = new StringBuffer();
        int start = 0;
        int end = tagged.indexOf(separator);
        while(end != -1) {
            result.append(tagged.substring(start,end));
            start=end+1;
            end= tagged.indexOf(separator,start);
            if (end != -1) {
                String tag = tagged.substring(start,end);
                String value = System.getProperty(tag);
                if (value == null)
                    result.append("@"+tag+"@");
                else {
                    TraceSystem.get("repository").debug("Replacing tag @"+tag+"@ with value "+value);
                    result.append(value);
                }
            }
            start = end+1;
            end = tagged.indexOf(separator,start);
        }
        result.append(tagged.substring(start));
        return result.toString();
    }

    /**
     * Loads the classpath specified in the classpath parameter.
     * @param classpath the classpath to load (space-separated).
     * @param description the context description to complete.
     */
    protected void loadClasspath(String classpath, ContextDescription description) {
       for (StringTokenizer st = new StringTokenizer(classpath!=null?classpath:"");st.hasMoreTokens();){
            String path = st.nextToken();
            if (!description.checkDescription(path)) {
                description.addDescription(path);
                TraceSystem.get("repository").debug("Loading path "+path+" ...");
                Classpath cl = (Classpath)getDescriptions().get(path);
                if (cl != null)
                    loadClasspath(cl, description);
                else
                    throw new LauncherException(new Exception("Path " + path + " not found!"));
            }
        }

    }

    /**
     * Loads the content of the classpath into the description.
     * @param classpath the classpath to load.
     * @param description the description where classpath should be loaded.
     */
    protected void loadClasspath(Classpath classpath, ContextDescription description) {
        if (classpath == null)
            throw new LauncherException(new Exception("No path found!"));
        for (Iterator i = classpath.getPathList().iterator() ; i.hasNext() ; ) {
            Path path = (Path)i.next();
            if (path.getUrl() != null)
                description.addClass(path.getUrl());
            else
                loadClasspath(path.getClasspath(), description);
        }
    }

    /**
     * Loads the list of arguments specified in the args parameter.
     * @param args the list of arguments to load (space-separated).
     * @param description the context description to complete.
     */
    protected void loadArguments(String args, ContextDescription description) {
        for (StringTokenizer st = new StringTokenizer(args!=null?args:"");st.hasMoreTokens();){
            String arg = st.nextToken();
            if (!description.checkDescription(arg)) {
                description.addDescription(arg);
                TraceSystem.get("repository").debug("Loading argument "+arg+" ...");
                Arguments arguments = (Arguments)getDescriptions().get(arg);
                if (arguments != null)
                    loadArguments(arguments, description);
                else
                    throw new LauncherException(new Exception("Argument " + arg + " not found!"));
            }
        }
    }

    /**
     * Loads the content of the arguments into the description.
     * @param args the arguments to load.
     * @param description the description where arguments should be loaded.
     */
    protected void loadArguments(Arguments args, ContextDescription description) {
        if (args == null)
            throw new LauncherException(new Exception("No argument found!"));
        for(Iterator i = args.getArgumentList().iterator() ; i.hasNext() ; ) {
            Argument arg = (Argument)i.next();
            if (arg.getValue() != null) {
                String toAdd = replaceTag(arg.getValue());
                description.addArgument(toAdd);
            } else if (arg.getLine() != null){
                String toAdd = replaceTag(arg.getLine());
                for (StringTokenizer st = new StringTokenizer(toAdd);st.hasMoreTokens();)
                    description.addArgument(st.nextToken());
            } else
                loadArguments(arg.getArguments(), description );
        }
    }

    /**
     * Loads the list of properties specified in the props parameter.
     * @param props this list of properties to load (space-separated).
     * @param description the context description to complete.
     */
    protected void loadProperties(String props, ContextDescription description) {
        for (StringTokenizer st = new StringTokenizer(props!=null?props:"");st.hasMoreTokens();){
            String prop = st.nextToken();
            if (!description.checkDescription(prop)) {
                description.addDescription(prop);
                TraceSystem.get("repository").debug("Loading property "+prop+" ...");
                Properties properties = (Properties)getDescriptions().get(prop);
                if (properties != null)
                    loadProperties(properties, description);
                else
                    throw new LauncherException(new Exception("Property " + prop + " not found!"));
            }
        }
    }

    /**
     * Loads the content of the properties into the description.
     * @param props the properties to load.
     * @param description the description where properties should be loaded.
     */
    protected void loadProperties(Properties props, ContextDescription description) {
        if (props == null)
            throw new LauncherException(new Exception("No property found!"));
        for(Iterator i = props.getPropertyList().iterator() ; i.hasNext() ; ) {
            Property prop = (Property)i.next();
            if ((prop.getName() != null) && (prop.getValue() != null))
                description.addProperty(prop.getName(), replaceTag(prop.getValue()));
            else
                loadProperties(prop.getProperties(), description);
        }
    }


    /**
     * Loads the list of contexts specified in the ctx parameter.
     * @param props this list of contexts to load (space-separated).
     * @param description the context description to complete.
     */
    protected void loadContexts(String ctx, ContextDescription description) {
        for (StringTokenizer st = new StringTokenizer(ctx!=null?ctx:"");st.hasMoreTokens();){
            String context = st.nextToken();
            if (!description.checkDescription(context)) {
                description.addDescription(context);
                TraceSystem.get("repository").debug("Loading context "+context+" ...");
                Context context_obj = (Context)getDescriptions().get(context);
                if (context_obj != null)
                    loadContexts(context_obj, description);
                else
                    throw new LauncherException(new Exception("Context " + context + " not found!"));
            }
        }
    }

    /**
     * Loads the content of the context into the description.
     * @param ctx the context to load.
     * @param description the description where properties should be loaded.
     */
    protected void loadContexts(Context ctx, ContextDescription description) {
        if (ctx == null)
            throw new LauncherException(new Exception("No context found!"));
        description.setName(ctx.getId());
        loadClasspath(ctx.getClasspath(),description);
        loadArguments(ctx.getArguments(),description);
        loadProperties(ctx.getProperties(),description);
        loadContexts(ctx.getContexts(),description);
    }

    /**
     * Loads the content of the run into the description.
     * @param run the run to load.
     * @param description the description where properties should be loaded.
     */
    protected void loadRun(Run run, RunDescription description) {
       if (run == null)
           throw new LauncherException(new Exception("No run found!"));
       description.setMainclass(run.getMainclassname());
       description.setName(run.getId());
       loadClasspath(run.getClasspath(),description);
       loadArguments(run.getArguments(),description);
       loadProperties(run.getProperties(),description);
       loadContexts(run.getContexts(),description);
   }

    /**
     * Complete an existing description with the content of a context description.
     * @param identifier the identifier of the context description.
     * @param description the description to complete.
     */
   public void completeDescription(String identifier, ContextDescription description) {
       loadContexts((Context)getDescriptions().get(identifier), description);
   }

    /**
     * Retrieves a description corresponding to the identifier parameter.
     * @param identifier the identifier of the description to obtain.
     * @return the description.
     */
    public ContextDescription getDescription(String identifier) {
        Object element = getDescriptions().get(identifier);
        if (element instanceof Run) {
            RunDescription desc = new RunDescription();
            loadRun((Run)element,desc);
            return desc;
        } else if (element instanceof Context) {
            ContextDescription desc = new ContextDescription();
            loadContexts((Context)element,desc);
            return desc;
        } else
            throw new LauncherException(new Exception("No description found for " + identifier + "!"));
    }
}
