/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: TraceSystem.java,v 1.1 2004/02/13 17:46:08 rouvoy Exp $
====================================================================*/

package org.objectweb.util.trace;


import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.objectweb.util.monolog.api.HandlerFactory;
import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.file.monolog.PropertiesConfAccess;



/**
 * Specialization of the Monolog trace mechanism into a TraceSystem
 * for dealing with trace in the scope of the Launcher.<BR>
 * <p>
 *   The TraceSystem provides a set of preconfigured loggers (through 
 *   the default.properties file) for enabling trace in the launcher.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class TraceSystem
{
    /** prefix used to identify Launcher loggers */
    protected static final String prefix = "org.objectweb.util";

    /** list of current logger instancied */
    protected static Map list_ = new HashMap();

    /** default level of log */
    protected static String level_ = "default";

    /**
     * Configure the log for Launcher
     * Log configuration is stored in a property file, <code>trace.properties</code>,
     * which should be available from the classpath.
     */
    protected static LoggerFactory getLoggerFactory() {
        Properties props = new Properties();
        try {
            props.load(Thread.currentThread().getContextClassLoader().getResourceAsStream(level_+".properties"));
        } catch (Exception e) {
            System.err.println("Configuration file for log not found. Traces are disabled: " + e);
            return null;
        }
        
        // Instanciate the LoggerFactory
        String lfClassName = props.getProperty("log.config.classname", null);
        if (lfClassName == null) {
            System.err.println("Malformed log configuration file: log.config.classname not available");
        }

        try {
            LoggerFactory lf = (LoggerFactory) Class.forName(lfClassName).newInstance();
            // Configure the LoggerFactory with the properties
            PropertiesConfAccess.load(props, lf, (HandlerFactory) lf, (LevelFactory) lf);
            return lf;
        } catch (Exception e) {
            System.err.println("Logs are disabled:" + e);
        }

        return null;
    }

    /**
     * Creates a new logger identified by logger value
     *
     * @param logger - the name of the logger
     */
    protected static void create(String logger) {        
        list_.put(logger,
                  new TraceTemplate(getLoggerFactory().getLogger(prefix+"."+logger)));
    }

    /**
     * Changes the level of log
     *
     * @param level - the name of the level
     */
    public static void setLevel(String level) { 
        level_ = level; 
    }

    /**
     * Retrieve the instance of a logger
     *
     * @param id - the identifier of the logger
     */
    public static Trace get(String id) {
        if (!list_.containsKey(id))
            create(id);
        return (Trace) list_.get(id); 
    }
}
